@@ -1,4 +1,4 @@
-# $Id: crossgcc-1.0.tcl 100353 2012-12-09 02:37:02Z raimue@macports.org $
+# $Id: crossgcc-1.0.tcl 112828 2013-11-02 12:32:59Z raimue@macports.org $
 #
 # Copyright (c) 2011 The MacPorts Project
 # All rights reserved.
@@ -145,6 +145,11 @@ proc crossgcc.setup {target version} {
         configure.objc_archflags
         configure.ld_archflags
 
+        # the bootstrap compiler doesn't accept -stdlib
+        if {[info exists configure.cxx_stdlib]} {
+            configure.cxx_stdlib
+        }
+
         # We don't need system includes(this prevents xgcc to include system-wide
         # unwind.h if it is present)!
         compiler.cpath
