@@ -1,4 +1,4 @@
-# $Id: haskell-1.0.tcl 106872 2013-06-09 13:23:06Z cal@macports.org $
+# $Id: haskell-1.0.tcl 114507 2013-12-09 22:56:18Z cal@macports.org $
 #
 # Copyright (c) 2009 The MacPorts Project
 # All rights reserved.
@@ -53,7 +53,7 @@ array set haskell.compiler_configuration {
 proc haskell.setup {package version {compiler ghc} {register_scripts "yes"}} {
     global haskell.compiler_list
     global haskell.compiler_configuration
-    global homepage prefix configure.cmd destroot worksrcpath name master_sites configure.cc
+    global homepage prefix configure.cmd destroot worksrcpath name master_sites configure.cc extract.suffix
 
     if {![info exists haskell.compiler_configuration($compiler)]} {
         return -code error "Compiler ${compiler} not currently supported"
@@ -63,7 +63,7 @@ proc haskell.setup {package version {compiler ghc} {register_scripts "yes"}} {
     version             ${version}
     categories          devel haskell
     homepage            http://hackage.haskell.org/package/${package}
-    master_sites        http://hackage.haskell.org/packages/archive/${package}/${version}
+    master_sites        http://hackage.haskell.org/package/${package}-${version}
     distname            ${package}-${version}
     depends_lib         port:${compiler_config(port)}
     configure.cmd       runhaskell
@@ -92,13 +92,14 @@ proc haskell.setup {package version {compiler ghc} {register_scripts "yes"}} {
 			system "${prefix}/libexec/${name}/register.sh"
 		}
 		pre-deactivate {
-			system "${prefix}/libexec/${name}/unregister.sh"
+			# deactivate even if that would break dependent ports; let MacPorts deal with this problem
+			system "${prefix}/libexec/${name}/unregister.sh --force"
 		}
 	}
 
     livecheck.type      regex
-    livecheck.url       http://hackage.haskell.org/cgi-bin/hackage-scripts/package/${package}
-    livecheck.regex     /packages/archive/${package}/.*/${package}-(.*)\.tar\.gz
+    livecheck.url       http://hackage.haskell.org/package/${package}
+    livecheck.regex     "/package/[quotemeta ${package}]-\[^/\]+/[quotemeta ${package}]-(\[^\"\]+)[quotemeta ${extract.suffix}]"
 
     universal_variant   no
 }
