@@ -1,9 +1,7 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
-# muniversal-1.0.tcl
+# $Id: muniversal-1.0.tcl 115752 2014-01-10 20:33:33Z jeremyhu@macports.org $
 #
-# $Id: muniversal-1.0.tcl 68598 2010-06-07 15:54:08Z jmr@macports.org $
-#
-# Copyright (c) 2009 The MacPorts Project,
+# Copyright (c) 2009-2013 The MacPorts Project,
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -18,7 +16,7 @@
 # 3. Neither the name of Apple Computer, Inc. nor the names of its
 #    contributors may be used to endorse or promote products derived from
 #    this software without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 # "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 # LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -35,9 +33,12 @@
 # User variables:
 #         merger_configure_env: associative array of configure.env variables
 #             merger_build_env: associative array of build.env variables
+#          merger_destroot_env: associative array of destroot.env variables
 #                  merger_host: associative array of host values
 #        merger_configure_args: associative array of configure.args
 #            merger_build_args: associative array of build.args
+#         merger_destroot_args: associative array of destroot.args
+#    merger_configure_compiler: associative array of configure.compiler
 #    merger_configure_cppflags: associative array of configure.cppflags
 #      merger_configure_cflags: associative array of configure.cflags
 #    merger_configure_cxxflags: associative array of configure.cxxflags
@@ -45,7 +46,6 @@
 #     merger_configure_ldflags: associative array of configure.ldflags
 #             merger_arch_flag: if no, -arch xxx will not be appended configure.???flags
 #         merger_arch_compiler: if no, -arch xxx will not be appended to compilers
-#          merger_destroot_env: associative array of destroot.env variables
 #             merger_dont_diff: list of file names for which diff will not work
 #     merger_must_run_binaries: if yes, build platform must be able to run binaries for supported architectures
 #            merger_no_3_archs: if yes, merger will not work correctly if there are three supported architectures
@@ -59,19 +59,7 @@ default merger_arch_compiler {no}
 
 proc muniversal_arch_flag_supported {args} {
     global configure.compiler
-    switch -exact ${configure.compiler} {
-        gcc-4.0 -
-        gcc-4.2 -
-        llvm-gcc-4.2 -
-        clang -
-        apple-gcc-4.0 -
-        apple-gcc-4.2 {
-            return yes
-        }
-        default {
-            return no
-        }
-    }
+    return [regexp {^gcc-4|llvm|apple|clang} ${configure.compiler}]
 }
 
 proc muniversal_get_arch_flag {arch {fortran ""}} {
@@ -99,6 +87,34 @@ proc muniversal_get_arch_flag {arch {fortran ""}} {
     return ${archf}
 }
 
+# set up the merger-post-destroot hook
+
+proc merger_target_provides {ditem args} {
+    global targets
+    # register just the procedure, no pre-/post-
+    # User-code exceptions are caught and returned as a result of the target.
+    # Thus if the user code breaks, dependent targets will not execute.
+    foreach target $args {
+        set origproc [ditem_key $ditem procedure]
+        set ident [ditem_key $ditem name]
+        proc merger-post-$target {args} "
+            variable proc_index
+            set proc_index \[llength \[ditem_key $ditem post\]\]
+            ditem_append $ditem merger-post proc-merger-post-${ident}-${target}-\${proc_index}
+            proc proc-merger-post-${ident}-${target}-\${proc_index} {name} \"
+                if {\\\[catch userproc-merger-post-${ident}-${target}-\${proc_index} result\\\]} {
+                    return -code error \\\$result
+                } else {
+                    return 0
+                }
+            \"
+            makeuserproc userproc-merger-post-${ident}-${target}-\${proc_index} \$args
+        "
+    }
+}
+
+merger_target_provides ${org.macports.destroot} destroot
+
 variant universal {
     global universal_archs_to_use
 
@@ -119,9 +135,15 @@ variant universal {
     if { ${merger_must_run_binaries}=="yes" } {
         if { ${os.arch}=="i386" } {
             set universal_archs_supported [ldelete ${universal_archs_supported} "ppc64"]
+            if {${os.major} >= 9 && [sysctl hw.cpu64bit_capable] == 0} {
+                set universal_archs_supported [ldelete ${universal_archs_supported} "x86_64"]
+            }
         } else {
             set universal_archs_supported [ldelete ${universal_archs_supported} "i386"]
             set universal_archs_supported [ldelete ${universal_archs_supported} "x86_64"]
+            if {${os.major} >= 9 && [sysctl hw.cpu64bit_capable] == 0} {
+                set universal_archs_supported [ldelete ${universal_archs_supported} "ppc64"]
+            }
         }
     }
 
@@ -164,10 +186,15 @@ variant universal {
     }
 
     configure {
-        foreach arch ${universal_archs_to_use} {
-            ui_info "$UI_PREFIX [format [msgcat::mc "Configuring %1\$s for architecture %2\$s"] $name ${arch}]"
+        # Fix inability to find nm when cross-compiling (#22224, #23431, #23687, #24477, et al)
+        configure.env-append    NM=/usr/bin/nm
 
-            copy ${worksrcpath} ${worksrcpath}-${arch}
+        foreach arch ${universal_archs_to_use} {
+            ui_info "$UI_PREFIX [format [msgcat::mc "Configuring %1\$s for architecture %2\$s"] ${subport} ${arch}]"
+
+            if {![file exists ${worksrcpath}-${arch}]} {
+                copy ${worksrcpath} ${worksrcpath}-${arch}
+            }
 
             set archf [muniversal_get_arch_flag ${arch}]
             set archff [muniversal_get_arch_flag ${arch} "fortran"]
@@ -203,43 +230,53 @@ variant universal {
 
             # Don't set the --host unless we have to.
             set host ""
-            if { ${os.arch}=="i386" && (${arch}=="ppc" || ${arch}=="ppc64") } {
-                if { [info exists merger_host(${arch})] } {
-                    if { $merger_host(${arch}) != "" } {
-                        set host  --host=$merger_host(${arch})
-                    }
-                } else {
-                    if { ${arch}=="ppc" } {
-                        set host --host=powerpc-apple-darwin${os.version}
-                    } else {
-                        set host --host=powerpc64-apple-darwin${os.version}
-                    }
+            if { [info exists merger_host($arch)] } {
+                if { $merger_host($arch) != "" } {
+                    set host  --host=$merger_host($arch)
                 }
-            } elseif { ${os.arch}=="powerpc" && (${arch}=="i386" || ${arch}=="x86_64") } {
-                if { [info exists merger_host(${arch})] } {
-                    if { $merger_host(${arch}) != "" } {
-                        set host  --host=$merger_host(${arch})
-                    }
-                } else {
-                    if { ${arch}=="i386" } {
-                        set host --host=i386-apple-darwin${os.version}
-                    } else {
-                        set host --host=x86_64-apple-darwin${os.version}
+            } elseif {[file tail ${configure.cmd}] != "cmake"} {
+                # check if building for a word length we can't run
+                set bits_differ 0
+                if {(${arch}=="x86_64" || ${arch}=="ppc64") &&
+                    (${os.major} < 9 || [sysctl hw.cpu64bit_capable] == 0)} {
+                    set bits_differ 1
+                }
+                # check if building for a completely different arch
+                if {$bits_differ || (${os.arch}=="i386" && (${arch}=="ppc" || ${arch}=="ppc64"))
+                        || (${os.arch}=="powerpc" && (${arch}=="i386" || ${arch}=="x86_64"))} {
+                    switch -- ${arch} {
+                        x86_64  {set host "--host=x86_64-apple-${os.platform}${os.version}"}
+                        i386    {set host "--host=i686-apple-${os.platform}${os.version}"}
+                        ppc     {set host "--host=powerpc-apple-${os.platform}${os.version}"}
+                        ppc64   {set host "--host=powerpc64-apple-${os.platform}${os.version}"}
                     }
                 }
             }
-            configure.args-append  ${host}
+            if {$host != ""} {
+                configure.args-append  ${host}
+            }
 
             if { [info exists merger_configure_args(${arch})] } {
                 configure.args-append  $merger_configure_args(${arch})
             }
 
-            set configure_cc_save   ${configure.cc}
-            set configure_cxx_save  ${configure.cxx}
-            set configure_objc_save ${configure.objc}
-            set configure_fc_save   ${configure.fc}
-            set configure_f77_save  ${configure.f77}
-            set configure_f90_save  ${configure.f90}
+            set configure_compiler_save ${configure.compiler}
+            set configure_cc_save       ${configure.cc}
+            set configure_cxx_save      ${configure.cxx}
+            set configure_objc_save     ${configure.objc}
+            set configure_fc_save       ${configure.fc}
+            set configure_f77_save      ${configure.f77}
+            set configure_f90_save      ${configure.f90}
+
+            if { [info exists merger_configure_compiler($arch)] } {
+                configure.compiler  $merger_configure_compiler($arch)
+                configure.cc        [portconfigure::configure_get_compiler cc]
+                configure.cxx       [portconfigure::configure_get_compiler cxx]
+                configure.objc      [portconfigure::configure_get_compiler objc]
+                configure.f77       [portconfigure::configure_get_compiler f77]
+                configure.f90       [portconfigure::configure_get_compiler f90]
+                configure.fc        [portconfigure::configure_get_compiler fc]
+            }
 
             if { ${merger_arch_compiler} != "no" } {
                 configure.cc   ${configure.cc}   ${archf}
@@ -279,6 +316,7 @@ variant universal {
             # Undo changes to the configure related variables
             eval autoreconf.dir ${autoreconf_dir_save}
             eval configure.dir  ${configure_dir_save}
+            eval configure.compiler ${configure_compiler_save}
             eval configure.f90  ${configure_f90_save}
             eval configure.f77  ${configure_f77_save}
             eval configure.fc   ${configure_fc_save}
@@ -321,8 +359,8 @@ variant universal {
 
     build {
         foreach arch ${universal_archs_to_use} {
-            ui_info "$UI_PREFIX [format [msgcat::mc "Building %1\$s for architecture %2\$s"] $name ${arch}]"
-            
+            ui_info "$UI_PREFIX [format [msgcat::mc "Building %1\$s for architecture %2\$s"] ${subport} ${arch}]"
+
             if { [info exists merger_build_env(${arch})] } {
                 build.env-append  $merger_build_env(${arch})
             }
@@ -355,7 +393,7 @@ variant universal {
 
     destroot {
         foreach arch ${universal_archs_to_use} {
-            ui_info "$UI_PREFIX [format [msgcat::mc "Staging %1\$s into destroot for architecture %2\$s"] $name ${arch}]"
+            ui_info "$UI_PREFIX [format [msgcat::mc "Staging %1\$s into destroot for architecture %2\$s"] ${subport} ${arch}]"
             copy ${destroot} ${workpath}/destroot-${arch}
             set destdirSave ${destroot.destdir}
             eval destroot.destdir  [string map "${destroot} ${workpath}/destroot-${arch}" ${destroot.destdir}]
@@ -363,6 +401,9 @@ variant universal {
             if { [info exists merger_destroot_env(${arch})] } {
                 destroot.env-append  $merger_destroot_env(${arch})
             }
+            if { [info exists merger_destroot_args(${arch})] } {
+                destroot.args-append  $merger_destroot_args(${arch})
+            }
             set destroot_dir_save ${destroot.dir}
             if { [string match "${worksrcpath}/*" ${destroot.dir}] } {
                 # The destroot directory is inside the source directory, so put in the new source directory name.
@@ -378,6 +419,9 @@ variant universal {
             portdestroot::destroot_main
 
             destroot.dir  ${destroot_dir_save}
+            if { [info exists merger_destroot_args(${arch})] } {
+                destroot.args-delete $merger_destroot_args(${arch})
+            }
             if { [info exists merger_destroot_env(${arch})] } {
                 destroot.env-delete  $merger_destroot_env(${arch})
             }
@@ -385,6 +429,51 @@ variant universal {
         }
         delete ${destroot}
 
+        # execute merger-post-destroot, if it exists
+
+        set ditem ${org.macports.destroot}
+        set procedure [ditem_key $ditem merger-post]
+        if {$procedure != ""} {
+            set targetname [ditem_key $ditem name]
+            ui_debug "Executing org.macports.merger-post-destroot"
+            set result [catch { $procedure $targetname } errstr]
+            # Save variables in order to re-throw the same error code.
+            set errcode $::errorCode
+            set errinfo $::errorInfo
+            if {$result != 0} {
+                set portname $subport
+                ui_error "$targetname for port $portname returned: $errstr"
+                ui_debug "Error code: $errcode"
+                ui_debug "Backtrace: $errinfo"
+                return $result
+            }
+        }
+
+        # Merge two files (${dir1}/${fl} and ${dir2}/${fl}) to ${dir}/${fl}
+        # by stripping out -arch XXXX, -m32, and -m64
+        proc mergeStripArchFlags {dir1 dir2 dir fl} {
+            set tempdir [mkdtemp "/tmp/muniversal.XXXXXXXX"]
+            set tempfile1 "${tempdir}/1-${fl}"
+            set tempfile2 "${tempdir}/2-${fl}"
+
+            copy ${dir1}/${fl} ${tempfile1}
+            copy ${dir2}/${fl} ${tempfile2}
+
+            reinplace -E {s:-arch +[^ ]+::g} ${tempfile1} ${tempfile2}
+            reinplace {s:-m32::g} ${tempfile1} ${tempfile2}
+            reinplace {s:-m64::g} ${tempfile1} ${tempfile2}
+
+            if { ! [catch {system "/usr/bin/cmp -s \"${tempfile1}\" \"${tempfile2}\""}] } {
+                # modified files are identical
+                ui_debug "universal: merge: ${fl} differs in ${dir1} and ${dir2} but are the same when stripping out -m32, -m64, and -arch XXX"
+                copy ${tempfile1} ${dir}/${fl}
+                delete ${tempfile1} ${tempfile2} ${tempdir}
+            } else {
+                delete ${tempfile1} ${tempfile2} ${tempdir}
+                return -code error "${fl} differs in ${dir1} and ${dir2} and cannot be merged"
+            }
+        }
+
         # Merge ${base1}/${prefixDir} and ${base2}/${prefixDir} into dir ${base}/${prefixDir}
         #        arch1, arch2: names to prepend to files if a diff merge of two files is forbidden by merger_dont_diff
         #    merger_dont_diff: list of files for which /usr/bin/diff ${diffFormat} will not merge correctly
@@ -396,15 +485,21 @@ variant universal {
 
             xinstall -d -m 0755 ${dir}
 
-            foreach fl [glob -directory ${dir2} -tails -nocomplain *] {
-                if { ![file exists ${dir1}/${fl}] } {
+            foreach fl [glob -directory ${dir2} -tails -nocomplain * .*] {
+                if { ${fl} == "." || ${fl} == ".." } {
+                    continue
+                }
+                if { ![muniversal_file_or_symlink_exists ${dir1}/${fl}] } {
                     # File only exists in ${dir1}
                     ui_debug "universal: merge: ${prefixDir}/${fl} only exists in ${base2}"
                     copy ${dir2}/${fl} ${dir}
                 }
             }
-            foreach fl [glob -directory ${dir1} -tails -nocomplain *] {
-                if { ![file exists ${dir2}/${fl}] } {
+            foreach fl [glob -directory ${dir1} -tails -nocomplain * .*] {
+                if { ${fl} == "." || ${fl} == ".." } {
+                    continue
+                }
+                if { ![muniversal_file_or_symlink_exists ${dir2}/${fl}] } {
                     # File only exists in ${dir2}
                     ui_debug "universal: merge: ${prefixDir}/${fl} only exists in ${base1}"
                     copy ${dir1}/${fl} ${dir}
@@ -432,17 +527,20 @@ variant universal {
                         merge2Dir ${base1} ${base2} ${base} ${prefixDir}/${fl} ${arch1} ${arch2} ${merger_dont_diff} ${diffFormat}
                     } else {
                         # Files are neither directories nor links
-                        if { ! [catch {system "/usr/bin/cmp ${dir1}/${fl} ${dir2}/${fl} && /bin/cp -v ${dir1}/${fl} ${dir}"}] } {
+                        if { ! [catch {system "/usr/bin/cmp -s \"${dir1}/${fl}\" \"${dir2}/${fl}\" && /bin/cp -v \"${dir1}/${fl}\" \"${dir}\""}] } {
                             # Files are byte by byte the same
                             ui_debug "universal: merge: ${prefixDir}/${fl} is identical in ${base1} and ${base2}"
                         } else {
                             # Actually try to merge the files
-                            # First try lipo
-                            if { ! [catch {system "/usr/bin/lipo -create ${dir1}/${fl} ${dir2}/${fl} -output ${dir}/${fl}"}] } {
+                            # First try lipo, then libtool
+                            if { ! [catch {system "/usr/bin/lipo -create \"${dir1}/${fl}\" \"${dir2}/${fl}\" -output \"${dir}/${fl}\""}] } {
                                 # lipo worked
                                 ui_debug "universal: merge: lipo created ${prefixDir}/${fl}"
+                            } elseif { ! [catch {system "/usr/bin/libtool \"${dir1}/${fl}\" \"${dir2}/${fl}\" -o \"${dir}/${fl}\""}] } {
+                                # libtool worked
+                                ui_debug "universal: merge: libtool created ${prefixDir}/${fl}"
                             } else {
-                                # lipo has failed, so assume they are text files to be merged
+                                # lipo and libtool have failed, so assume they are text files to be merged
                                 set dontdiff no
                                 foreach dont ${merger_dont_diff} {
                                     if { ${dont}=="${prefixDir}/${fl}" } {
@@ -463,20 +561,17 @@ variant universal {
 
                                     ui_debug "universal: merge: created ${prefixDir}/${fl} to include ${prefixDir}/${arch1}-${fl} ${prefixDir}/${arch1}-${fl}"
 
-                                    system "/usr/bin/diff -d ${diffFormat} ${dir}/${arch1}-${fl} ${dir}/${arch2}-${fl} > ${dir}/${fl}; test \$? -le 1"
+                                    system "/usr/bin/diff -d ${diffFormat} \"${dir}/${arch1}-${fl}\" \"${dir}/${arch2}-${fl}\" > \"${dir}/${fl}\"; test \$? -le 1"
 
                                     copy -force ${dir1}/${fl} ${dir}/${arch1}-${fl}
                                     copy -force ${dir2}/${fl} ${dir}/${arch2}-${fl}
                                 } else {
-                                    set known_file "no"
-
                                     # Text file on which diff will not give correct results.
                                     switch -glob ${fl} {
                                         *.mod {
                                             # .mod files from Fortran modules.
                                             # Create a sepcial module directory for each architecture.
                                             # To find these modules, GFortran might require -M or -J.
-                                            set known_file "yes"
                                             file mkdir ${dir}/mods32
                                             file mkdir ${dir}/mods64
                                             if { ${arch1}=="i386" || ${arch1}=="ppc" } {
@@ -487,76 +582,88 @@ variant universal {
                                                 copy ${dir1}/${fl} ${dir}/mods64
                                             }
                                         }
-                                        *.la -
                                         *.pc -
                                         *-config {
-                                            return -code error "${prefixDir}/${fl} differs in ${base1} and ${base2} and cannot be merged"
+                                            mergeStripArchFlags ${dir1} ${dir2} ${dir} ${fl}
                                         }
-                                    }
-
-                                    if { ${known_file}=="no" } {
-                                        if { ! [catch {system "/usr/bin/diff -dw ${diffFormat} ${dir1}/${fl} ${dir2}/${fl} > ${dir}/${fl}; test \$? -le 1"} ] } {
-                                            # diff worked
-                                            ui_debug "universal: merge: used diff to create ${prefixDir}/${fl}"
-                                        } else {
-                                            # File created by diff is invalid
-                                            delete ${dir}/${fl}
-                                            
-                                            # nothing has worked so far.
+                                        *.la {
+                                            global destroot.delete_la_files
+                                            if {${destroot.delete_la_files} == "yes"} {
+                                                ui_debug "universal: merge: ${prefixDir}/${fl} differs in ${base1} and ${base2}; ignoring due to delete_la_files"
+                                            } else {
+                                                return -code error "${prefixDir}/${fl} differs in ${base1} and ${base2} and cannot be merged"
+                                            }
+                                        }
+                                        *.typelib {
+                                            # Sometimes garbage ends up in ignored trailing bytes
+                                            # https://trac.macports.org/ticket/39629
+                                            # TODO: Compare the g-ir-generate output
+                                            ui_debug "universal: merge: ${prefixDir}/${fl} differs in ${base1} and ${base2}; assume trivial difference"
+                                            copy ${dir1}/${fl} ${dir}
+                                        }
+                                        *.jar {
+                                            # jar files can be different because of timestamp
+                                            ui_debug "universal: merge: ${prefixDir}/${fl} differs in ${base1} and ${base2}; assume timestamp difference"
+                                            copy ${dir1}/${fl} ${dir}
+                                        }
+                                        *.elc {
+                                            # elc files can be different because they record when and where they were built.
+                                            ui_debug "universal: merge: ${prefixDir}/${fl} differs in ${base1} and ${base2}; assume trivial difference"
+                                            copy ${dir1}/${fl} ${dir}
+                                        }
+                                        *.el.gz -
+                                        *.el.bz2 {
+                                            # Emacs lisp files should be same across architectures
+                                            # the emacs package (and perhaps others) records the date of automatically generated el files
+                                            ui_debug "universal: merge: ${prefixDir}/${fl} differs in ${base1} and ${base2}; assume trivial difference"
+                                            copy ${dir1}/${fl} ${dir}
+                                        }
+                                        *.lzma -
+                                        *.xz -
+                                        *.gz -
+                                        *.bz2 {
+                                            # compressed files can differ due to entropy
                                             switch -glob ${fl} {
-                                                *.jar {
-                                                    # jar files can be different because of timestamp
-                                                    ui_debug "universal: merge: ${prefixDir}/${fl} differs in ${base1} and ${base2}; assume timestamp difference"
-                                                    copy ${dir1}/${fl} ${dir}
+                                                *.lzma {
+                                                    set cat ${prefix}/bin/lzcat
                                                 }
-                                                *.elc {
-                                                    # elc files can be different because they record when and where they were built.
-                                                    ui_debug "universal: merge: ${prefixDir}/${fl} differs in ${base1} and ${base2}; assume trivial difference"
-                                                    copy ${dir1}/${fl} ${dir}
+                                                *.xz {
+                                                    set cat ${prefix}/bin/xzcat
+                                                }
+                                                *.gz {
+                                                    set cat /usr/bin/gzcat
                                                 }
-                                                *.gz -
                                                 *.bz2 {
-                                                    # compressed files can differ due to entropy
-                                                    switch -glob ${fl} {
-                                                        *.gz {
-                                                            set cat /usr/bin/gzcat
-                                                        }
-                                                        *.bz2 {
-                                                            set cat /usr/bin/bzcat
-                                                        }
-                                                    }
-                                                    set tempdir [mkdtemp "/tmp/muniversal.XXXXXXXX"]
-                                                    set tempfile1 "${tempdir}/${arch1}-[file rootname ${fl}]"
-                                                    set tempfile2 "${tempdir}/${arch2}-[file rootname ${fl}]"
-                                                    system "${cat} ${dir1}/${fl} > ${tempfile1}"
-                                                    system "${cat} ${dir2}/${fl} > ${tempfile2}"
-                                                    set identical "no"
-                                                    if { ! [catch {system "/usr/bin/cmp -s ${tempfile1} ${tempfile2}"}] } {
-                                                        # files are identical
-                                                        ui_debug "universal: merge: ${prefixDir}/${fl} differs in ${base1} and ${base2} but the contents are the same"
-                                                        set identical "yes"
-                                                        copy ${dir1}/${fl} ${dir}
-                                                    }
-                                                    if { ${identical}=="no" } {
-                                                        switch -glob ${fl} {
-                                                            *.el.gz {
-                                                                # Emacs lisp files should be same across architectures
-                                                                # the emacs package (and perhaps others) records the date of automatically generated el files
-                                                                ui_debug "universal: merge: ${prefixDir}/${fl} differs in ${base1} and ${base2}; assume trivial difference"
-                                                                set identical "yes"
-                                                                copy ${dir1}/${fl} ${dir}
-                                                            }
-                                                        }
-                                                    }
-                                                    delete ${tempfile1} ${tempfile2} ${tempdir}
-                                                    if {${identical}=="no"} {
-                                                        return -code error "${prefixDir}/${fl} differs in ${base1} and ${base2} and cannot be merged"
-                                                    }
-                                                }
-                                                default {
-                                                    return -code error "${prefixDir}/${fl} differs in ${base1} and ${base2} and cannot be merged"
+                                                    set cat /usr/bin/bzcat
                                                 }
                                             }
+                                            set tempdir [mkdtemp "/tmp/muniversal.XXXXXXXX"]
+                                            set tempfile1 "${tempdir}/${arch1}-[file rootname ${fl}]"
+                                            set tempfile2 "${tempdir}/${arch2}-[file rootname ${fl}]"
+                                            system "${cat} \"${dir1}/${fl}\" > \"${tempfile1}\""
+                                            system "${cat} \"${dir2}/${fl}\" > \"${tempfile2}\""
+                                            if { ! [catch {system "/usr/bin/cmp -s \"${tempfile1}\" \"${tempfile2}\""}] } {
+                                                # files are identical
+                                                ui_debug "universal: merge: ${prefixDir}/${fl} differs in ${base1} and ${base2} but the contents are the same"
+                                                copy ${dir1}/${fl} ${dir}
+                                                delete ${tempfile1} ${tempfile2} ${tempdir}
+                                            } else {
+                                                delete ${tempfile1} ${tempfile2} ${tempdir}
+                                                return -code error "${prefixDir}/${fl} differs in ${base1} and ${base2} and cannot be merged"
+                                            }
+                                        }
+                                        default {
+                                            if { ! [catch {system "test `head -c2 ${dir1}/${fl}` == '#!'"}] } {
+                                                # Shell script, hopefully striping out arch flags works...
+                                                mergeStripArchFlags ${dir1} ${dir2} ${dir} ${fl}
+                                            } elseif { ! [catch {system "/usr/bin/diff -dw ${diffFormat} \"${dir1}/${fl}\" \"${dir2}/${fl}\" > \"${dir}/${fl}\"; test \$? -le 1"} ] } {
+                                                # diff worked
+                                                ui_debug "universal: merge: used diff to create ${prefixDir}/${fl}"
+                                            } else {
+                                                # File created by diff is invalid
+                                                delete ${dir}/${fl}
+                                                return -code error "${prefixDir}/${fl} differs in ${base1} and ${base2} and cannot be merged"
+                                            }
                                         }
                                     }
                                 }
@@ -571,15 +678,15 @@ variant universal {
         # See http://www.gnu.org/software/diffutils/manual/html_mono/diff.html#If-then-else
         # See http://www.gnu.org/software/diffutils/manual/html_mono/diff.html#Detailed%20If-then-else
         set diffFormatProc {--old-group-format='#if (defined(__ppc__) || defined(__ppc64__))
- %<#endif /* __ppc__ || __ppc64__ */
+ %<#endif
 ' \
 --new-group-format='#if defined (__i386__) || defined(__x86_64__)
-%>#endif /* __i386__ || __x86_64__ */
+%>#endif
 ' \
 --unchanged-group-format='%=' \
 --changed-group-format='#if (defined(__ppc__) || defined(__ppc64__))
-%<#else /* ! __ppc__ && ! __ppc64__ */
-%>#endif /* __ppc__ || __ppc64__ */
+%<#else
+%>#endif
 '}
 
         set diffFormatM "-D __LP64__"
@@ -598,7 +705,7 @@ variant universal {
             # Rosetta does not translate G5 instructions
             # PowerPC systems can't translate Intel instructions
             if { (${os.arch}=="i386" && ${arch}!="ppc64") || (${os.arch}=="powerpc" && ${arch}!="i386" && ${arch}!="x86_64") } {
-                ui_info "$UI_PREFIX [format [msgcat::mc "Testing %1\$s for architecture %2\$s"] $name ${arch}]"
+                ui_info "$UI_PREFIX [format [msgcat::mc "Testing %1\$s for architecture %2\$s"] ${subport} ${arch}]"
                 set test_dir_save ${test.dir}
                 if { [string match "${worksrcpath}/*" ${test.dir}] } {
                     # The test directory is inside the source directory, so put in the new source directory name.
@@ -618,3 +725,15 @@ variant universal {
         }
     }
 }
+
+# [muniversal_file_or_symlink_exists ${f}] tells you if ${f} exists. And unlike
+# [file exists ${f}], if used on a symlink, [muniversal_file_or_symlink_exists ${f}]
+# tells you about the symlink, not what it points to.
+proc muniversal_file_or_symlink_exists {f} {
+    # If [file type ${f}] throws an error, ${f} doesn't exist.
+    if {[catch {file type ${f}}]} {
+        return 0
+    }
+    # Otherwise, it does.
+    return 1
+}
