@@ -1,7 +1,5 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
-# perl5-1.0.tcl
-#
-# $Id: perl5-1.0.tcl 91840 2012-04-11 19:41:37Z jeremyhu@macports.org $
+# $Id: perl5-1.0.tcl 116112 2014-01-19 03:21:21Z jmr@macports.org $
 #
 # Copyright (c) 2004 Robert Shaw <rshaw@opendarwin.org>,
 #                    Toby Peterson <toby@opendarwin.org>
@@ -20,7 +18,7 @@
 # 3. Neither the name of Apple Computer, Inc. nor the names of its
 #    contributors may be used to endorse or promote products derived from
 #    this software without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 # "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 # LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -35,20 +33,26 @@
 #
 
 # portfile configuration options
-# perl5.branches: the major perl version supported by this module. A
+# perl5.branches: the major perl versions supported by this module. A
 #   subport will be created for each. e.g. p5.12-foo, p5.10-foo, ...
+# perl5.branches must be set in the portfile
 # perl5.default_branch: the branch used when you request p5-foo
 options perl5.default_branch perl5.branches
-default perl5.branches {"5.8 5.10 5.12 5.14"}
 default perl5.default_branch {[perl5_get_default_branch]}
+
 proc perl5_get_default_branch {} {
-    global prefix
-    # use whatever ${prefix}/bin/perl5 was chosen, and if none, fall back to 5.12
+    global prefix perl5.branches
+    # use whatever ${prefix}/bin/perl5 was chosen, and if none, fall back to 5.16
     if {![catch {set val [lindex [split [exec ${prefix}/bin/perl5 -V:version] {'}] 1]}]} {
-        return [join [lrange [split $val .] 0 1] .]
+        set ret [join [lrange [split $val .] 0 1] .]
     } else {
-        return 5.12
+        set ret 5.16
     }
+    # if the above default is not supported by this module, use the latest it does support
+    if {[info exists perl5.branches] && [lsearch -exact ${perl5.branches} $ret] == -1} {
+        set ret [lindex ${perl5.branches} end]
+    }
+    return $ret
 }
 
 proc perl5.extract_config {var {default ""}} {
@@ -62,7 +66,7 @@ proc perl5.extract_config {var {default ""}} {
 }
 
 # Set some variables.
-options perl5.version perl5.major perl5.arch perl5.lib perl5.archlib perl5.bin
+options perl5.version perl5.major perl5.arch perl5.lib perl5.bindir perl5.archlib perl5.bin
 default perl5.version {[perl5.extract_config version]}
 default perl5.major {${perl5.default_branch}}
 default perl5.arch {[perl5.extract_config archname ${os.platform}]}
@@ -70,12 +74,17 @@ default perl5.bin {${prefix}/bin/perl${perl5.major}}
 
 # define installation libraries as vendor location
 default perl5.lib {${prefix}/lib/perl5/vendor_perl/${perl5.version}}
+default perl5.bindir {${prefix}/libexec/perl${perl5.major}}
 default perl5.archlib {${perl5.lib}/${perl5.arch}}
 
 default livecheck.version {${perl5.moduleversion}}
 
 default configure.universal_args {}
 
+options perl5.link_binaries perl5.link_binaries_suffix
+default perl5.link_binaries yes
+default perl5.link_binaries_suffix {-${perl5.major}}
+
 # define these empty initially, they are set by perl5.setup arguments
 set perl5.module ""
 set perl5.moduleversion ""
@@ -123,7 +132,7 @@ proc perl5.setup {module vers {cpandir ""}} {
             }
         }
 
-        if {$subport == $name} {
+        if {$subport eq $name} {
             perl5.major
             distfiles
             supported_archs noarch
@@ -139,17 +148,22 @@ proc perl5.setup {module vers {cpandir ""}} {
     } else {
         depends_lib port:perl${perl5.default_branch}
     }
-    if {![string match p5-* $name] || $subport != $name} {
+    if {![string match p5-* $name] || $subport ne $name} {
         configure.cmd       ${perl5.bin}
         configure.env       PERL_AUTOINSTALL=--skipdeps
         configure.pre_args  Makefile.PL
-        configure.args      INSTALLDIRS=vendor
+        default configure.args {"INSTALLDIRS=vendor CC=\"${configure.cc}\" LD=\"${configure.cc}\""}
 
-        # CCFLAGS can be passed in to "configure" but it's not necessarilary inherited
-        # LDFLAGS can't be passed in (or if it can, it's not easy to figure out how)
+        # CCFLAGS can be passed in to "configure" but it's not necessarily inherited.
+        # LDFLAGS can't be passed in (or if it can, it's not easy to figure out how).
         post-configure {
-            system "find ${worksrcpath} -name Makefile | xargs /usr/bin/sed -i \"\" '/^CCFLAGS *=/s/$/ [get_canonical_archflags cc]/' \;"
-            system "find ${worksrcpath} -name Makefile | xargs /usr/bin/sed -i \"\" '/^OTHERLDFLAGS *=/s/$/ [get_canonical_archflags ld]/'"
+            fs-traverse file ${configure.dir} {
+                if {[file isfile ${file}] && [file tail ${file}] eq "Makefile"} {
+                    ui_info "Fixing flags in [string map "${configure.dir}/ {}" ${file}]"
+                    reinplace "/^CCFLAGS *=/s/$/ [get_canonical_archflags cc]/" ${file}
+                    reinplace "/^OTHERLDFLAGS *=/s/$/ [get_canonical_archflags ld]/" ${file}
+                }
+            }
         }
 
         test.run            yes
@@ -158,11 +172,18 @@ proc perl5.setup {module vers {cpandir ""}} {
 
         post-destroot {
             fs-traverse file ${destroot}${perl5.lib} {
-                if {[file tail ${file}] eq ".packlist"} {
-                    ui_info "Fixing packlist ${file}"
+                if {[file isfile ${file}] && [file tail ${file}] eq ".packlist"} {
+                    ui_info "Fixing paths in [string map "${destroot}${perl5.lib}/ {}" ${file}]"
                     reinplace -n "s|${destroot}||p" ${file}
                 }
             }
+            if {${perl5.link_binaries}} {
+                foreach bin [glob -nocomplain -tails -directory "${destroot}${perl5.bindir}" *] {
+                    if {[catch {file type "${destroot}${prefix}/bin/${bin}${perl5.link_binaries_suffix}"}]} {
+                        ln -s "${perl5.bindir}/${bin}" "${destroot}${prefix}/bin/${bin}${perl5.link_binaries_suffix}"
+                    }
+                }
+            }
         }
     }
 
@@ -171,18 +192,18 @@ proc perl5.setup {module vers {cpandir ""}} {
     livecheck.regex     _gaq.push\\(\\\["_setCustomVar",5,"Release","[quotemeta ${perl5.module}]-(\[^"\]+?)\"
 }
 
-# Switch from default MakeMaker-style routine to Module::Build-style
+# Switch from default MakeMaker-style routine to Module::Build-style.
 proc perl5.use_module_build {} {
     global perl5.bin destroot perl5.major
 
-    if {${perl5.major} == ""} {
+    if {${perl5.major} eq ""} {
         return
     }
 
     depends_lib-append  port:p${perl5.major}-module-build
 
     configure.pre_args  Build.PL
-    configure.args      installdirs=vendor
+    default configure.args {"installdirs=vendor --config cc=\"${configure.cc}\" --config ld=\"${configure.cc}\""}
 
     build.cmd           ${perl5.bin}
     build.pre_args      Build
@@ -197,7 +218,7 @@ proc perl5.use_module_build {} {
     destroot.destdir    destdir=${destroot}
 }
 
-# convert a floating point version to an dotted-integer one
+# Convert a floating-point version to a dotted-integer one.
 proc perl5_convert_version {vers} {
     set index [string first . $vers]
     set other_dot [string first . [string range $vers [expr $index + 1] end]]
