@@ -1,5 +1,5 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
-# $Id: ruby-1.0.tcl 105953 2013-05-11 03:23:06Z ryandesign@macports.org $
+# $Id: ruby-1.0.tcl 115566 2014-01-05 12:52:34Z kimuraw@macports.org $
 #
 # Copyright (c) 2002 Apple Computer, Inc.
 # Copyright (c) 2004 Robert Shaw <rshaw@opendarwin.org>
@@ -46,11 +46,11 @@
 #     build.cmd        ${ruby.bin}
 
 # options:
-#   ruby.branch: select ruby version. 1.8, 1.9 or 2.0.
+#   ruby.branch: select ruby version. 1.8, 1.9, 2.0 or 2.1.
 #   ruby.link_binaries: whether generate suffixed symlink under ${prefix}/bin
 #        or not.
 # values:
-#   ruby.bin, ruby.rdoc, ruby.gem: fullpath to commands for ${ruby.branch}.
+#   ruby.bin, ruby.rdoc, ruby.gem ruby.rake: fullpath to commands for ${ruby.branch}.
 #   ruby.suffix: suffix of portname. port:ruby${ruby.suffix} or
 #        port:rb${ruby.suffix}-foo.
 #   ruby.bindir: install location of commands without suffix from rb-foo.
@@ -68,25 +68,28 @@
 options ruby.default_branch
 default ruby.default_branch 1.8
 options ruby.branch
-options ruby.bin ruby.rdoc ruby.gem ruby.bindir ruby.gemdir ruby.suffix
+options ruby.bin ruby.rdoc ruby.gem ruby.rake ruby.bindir ruby.gemdir ruby.suffix
+options ruby.api_version ruby.lib ruby.archlib
+# ruby.version is obsoleted. use ruby.api_version.
+options ruby.version
 option_proc ruby.branch ruby_set_branch
 proc ruby_set_branch {option action args} {
     if {$action != "set"} {
         return
     }
     global prefix ruby.branch \
-           ruby.bin ruby.rdoc ruby.gem ruby.bindir ruby.gemdir \
-           ruby.suffix ruby.link_binaries_suffix ruby.api_version \
-           ruby.prog_suffix
+           ruby.bin ruby.rdoc ruby.gem ruby.rake ruby.bindir ruby.gemdir \
+           ruby.suffix ruby.link_binaries_suffix ruby.prog_suffix \
+           ruby.api_version ruby.lib ruby.archlib ruby.arch
     set ruby.bin            ${prefix}/bin/ruby${ruby.branch}
     set ruby.rdoc           ${prefix}/bin/rdoc${ruby.branch}
     set ruby.gem            ${prefix}/bin/gem${ruby.branch}
+    set ruby.rake           ${prefix}/bin/rake${ruby.branch}
     set ruby.bindir         ${prefix}/libexec/ruby${ruby.branch}
-    set ruby.gemdir         ${prefix}/lib/ruby${ruby.branch}/gems/${ruby.api_version}
-    # gem command for 1.8 from port:rb-rubygems
+    # gem, rake command for 1.8 from port:rb-rubygems, port:rb-rake
     if {${ruby.branch} eq "1.8"} {
         set ruby.gem        ${ruby.bindir}/gem
-        set ruby.gemdir     ${prefix}/lib/ruby/gems/${ruby.api_version}
+        set ruby.rake       ${ruby.bindir}/rake
     }
     set ruby.suffix         [join [split ${ruby.branch} .] {}]
     if {${ruby.branch} eq "1.8"} {
@@ -97,6 +100,18 @@ proc ruby_set_branch {option action args} {
     if {${ruby.branch} eq "1.8"} {
         set ruby.prog_suffix     ""
     }
+    #
+    switch -exact ${ruby.branch} {
+        1.8 {set ruby.api_version 1.8}
+        1.9 {set ruby.api_version 1.9.1}
+        2.0 {set ruby.api_version 2.0.0}
+        2.1 {set ruby.api_version 2.1.0}
+    }
+    set ruby.gemdir         ${prefix}/lib/ruby${ruby.prog_suffix}/gems/${ruby.api_version}
+    # define installation libraries as vendor location
+    default ruby.lib        {[ruby.extract_config vendorlibdir ${prefix}/lib/ruby${ruby.prog_suffix}/vendor_ruby/${ruby.api_version}]}
+    default ruby.archlib    {[ruby.extract_config vendorarchdir ${ruby.lib}/${ruby.arch}]}
+    set ruby.version        ${ruby.api_version}
 }
 
 proc ruby.extract_config {var {default ""}} {
@@ -107,47 +122,52 @@ proc ruby.extract_config {var {default ""}} {
     return $val
 }
 
-options ruby.api_version ruby.lib ruby.archlib
-default ruby.api_version    {[ruby.extract_config ruby_version]}
+options ruby.arch
 default ruby.arch           {[ruby.extract_config arch "${os.arch}-${os.platform}${os.major}"]}
-# define installation libraries as vendor location
-default ruby.lib            {[ruby.extract_config vendorlibdir ${prefix}/lib/ruby/vendor_ruby/${ruby.api_version}]}
-default ruby.archlib        {[ruby.extract_config vendorarchdir ${ruby.lib}/${ruby.arch}]}
-# ruby.version is obsoleted. use ruby.api_version.
-options ruby.version
-default ruby.version        {[ruby.extract_config ruby_version]}
 
 set ruby.module         ""
 set ruby.filename       ""
 set ruby.project        ""
 set ruby.docs           {}
 set ruby.srcdir         ""
+set ruby.prog_suffix    ""
 
 options ruby.link_binaries
 default ruby.link_binaries yes
 
+# detect setup.rb config option name of --rubyprog.
+# some setup.rb accepts this option by other name, such as --ruby-prog.
+# NOTE: set the value *before ruby.setup* to use ohter name.
+options ruby.config_rubyprog_name
+default ruby.config_rubyprog_name --rubyprog
+
 default ruby.branch         ${ruby.default_branch}
 
 # ruby group setup procedure; optional for ruby 1.8 if you want only
 # basic variables, like ruby.lib and ruby.archlib.
 proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {implementation "ruby"}} {
     global destroot prefix worksrcpath os.platform
-    global ruby.bin ruby.rdoc ruby.gem
+    global ruby.bin ruby.rdoc ruby.gem ruby.rake ruby.branch
     global ruby.api_version ruby.lib ruby.suffix ruby.bindir ruby.gemdir
     global ruby.module ruby.filename ruby.project ruby.docs ruby.srcdir
     global ruby.link_binaries_suffix
     # ruby.version is obsoleted. use ruby.gemdir.
     global ruby.prog_suffix
+    # from muniversal
+    global universal_archs_supported
+    global merger_configure_env merger_build_env merger_destroot_env
+    # for setup.rb +universal
+    global ruby.config_rubyprog_name
 
-    if {${implementation} eq "ruby19"} {
-        ruby.branch 1.9
-    } elseif {${implementation} eq "ruby20"} {
-        ruby.branch 2.0
-    } elseif {${implementation} eq "ruby"} {
-        ruby.branch 1.8
-    } else {
-        ui_error "ruby.setup: unknown implementation '${implementation}' specified (ruby, ruby19, ruby20 possible)"
-        return -code error "ruby.setup failed"
+    switch ${implementation} {
+        ruby21 { ruby.branch 2.1 }
+        ruby20 { ruby.branch 2.0 }
+        ruby19 { ruby.branch 1.9 }
+        ruby   { ruby.branch 1.8 }
+        default {
+            ui_error "ruby.setup: unknown implementation '${implementation}' specified (ruby, ruby19, ruby20, ruby21 possible)"
+            return -code error "ruby.setup failed"
+        }
     }
 
     # define ruby global names and lists
@@ -326,18 +346,49 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
             build.target        setup
 
             pre-destroot {
-                if {[file isfile ${worksrcpath}/config.save]} {
-                    reinplace "s|${prefix}|${destroot}${prefix}|g" \
-                        ${worksrcpath}/config.save
-                }
-                if {[file isfile ${worksrcpath}/.config]} {
-                    reinplace "s|${prefix}|${destroot}${prefix}|g" \
-                        ${worksrcpath}/.config
+                foreach conf {config.save .config} {
+                    if {[file isfile ${worksrcpath}/${conf}]} {
+                        reinplace "s|${prefix}|${destroot}${prefix}|g" \
+                            ${worksrcpath}/${conf}
+                    }
+                    if {[variant_isset universal]  && [info exists universal_archs_supported]} {
+                        foreach arch ${universal_archs_supported} {
+                            if {[file isfile ${worksrcpath}-${arch}/${conf}]} {
+                                reinplace "s|${prefix}|${destroot}-${arch}${prefix}|g" \
+                                    ${worksrcpath}-${arch}/${conf}
+                            }
+                        }
+                    }
                 }
             }
             destroot.cmd        ${ruby.bin} -rvendor-specific setup.rb
             destroot.target     install
             destroot.destdir
+            # extconf.rb|mkmf.rb of ruby-1.8 does not support universal binary.
+            # to build universal extentions, write "Portgrourp muniversal 1.0" in the Portfile.
+            if {[variant_isset universal] && (${ruby.branch} eq "1.8") && [info exists universal_archs_supported]} {
+                # generate wrapper for --rubyprog option
+                pre-configure {
+                    set fo [open ${worksrcpath}/_mp_arch_ruby w]
+                    puts $fo "#!/bin/sh"
+                    puts $fo "/usr/bin/arch ${ruby.bin} \$@"
+                    close $fo
+                    system "chmod +x ${worksrcpath}/_mp_arch_ruby"
+                }
+                configure.pre_args-append \
+                    ${ruby.config_rubyprog_name}=${worksrcpath}/_mp_arch_ruby
+                foreach arch ${universal_archs_supported} {
+                    lappend merger_configure_env(${arch}) \
+                        ARCHPREFERENCE=ruby${ruby.branch}:${arch}
+                    lappend merger_build_env(${arch}) \
+                        ARCHPREFERENCE=ruby${ruby.branch}:${arch}
+                    lappend merger_destroot_env(${arch}) \
+                        ARCHPREFERENCE=ruby${ruby.branch}:${arch}
+                }
+                configure.cmd   /usr/bin/arch ${ruby.bin} -rvendor-specific setup.rb
+                build.cmd       /usr/bin/arch ${ruby.bin} -rvendor-specific setup.rb
+                destroot.cmd    /usr/bin/arch ${ruby.bin} -rvendor-specific setup.rb
+            }
             post-destroot {
                 foreach file [readdir ${destroot}${prefix}/bin] {
                     move [file join ${destroot}${prefix}/bin $file] ${destroot}${ruby.bindir}
@@ -352,6 +403,16 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
             build.args          RUBY="${ruby.bin}"
 
             destroot.args       RUBY="${ruby.bin}"
+
+            # extconf.rb|mkmf.rb of ruby-1.8 does not support universal binary.
+            # to build universal extentions, write "Portgrourp muniversal 1.0" in the Portfile.
+            if {[variant_isset universal] && (${ruby.branch} eq "1.8") && [info exists universal_archs_supported]} {
+                foreach arch ${universal_archs_supported} {
+                    lappend merger_configure_env(${arch}) \
+                        ARCHPREFERENCE=ruby${ruby.branch}:${arch}
+                }
+                configure.cmd   /usr/bin/arch ${ruby.bin} extconf.rb
+            }
             post-destroot {
                 foreach file [readdir ${destroot}${prefix}/bin] {
                     move [file join ${destroot}${prefix}/bin $file] ${destroot}${ruby.bindir}
@@ -379,6 +440,9 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
 
             if {${implementation} eq "ruby"} {
                 depends_lib-append  port:rb-rubygems
+                if {${ruby.module} ne "rake"} {
+                    depends_build-append    port:rb-rake
+                }
             }
 
             extract {}
@@ -388,8 +452,16 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
                 xinstall -d -m 0755 ${destroot}${ruby.gemdir}
             }
 
+            destroot.cmd    ${ruby.gem}
+            destroot.target install
+            destroot.args   --local --force --install-dir ${destroot}${ruby.gemdir}
+            destroot.env-append rake=${ruby.rake}
+
             destroot {
-                system -W ${worksrcpath} "${ruby.gem} install --local --force --install-dir ${destroot}${ruby.gemdir} ${distpath}/${distname}.gem"
+                # note: port cannot read $distpath and $distname
+                #       outside of destroot {}
+                destroot.post_args ${distpath}/${distname}.gem
+                command_exec destroot
 
                 set binDir ${destroot}${ruby.gemdir}/bin
                 if {[file isdirectory $binDir]} {
