@@ -1,12 +1,13 @@
-# $Id: xcodeversion-1.0.tcl 67639 2010-05-15 11:14:57Z jmr@macports.org $
-# 
+# -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
+# $Id: xcodeversion-1.0.tcl 109954 2013-08-22 22:22:43Z raimue@macports.org $
+#
 # Copyright (c) 2009 The MacPorts Project
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
 # met:
-# 
+#
 # 1. Redistributions of source code must retain the above copyright
 #    notice, this list of conditions and the following disclaimer.
 # 2. Redistributions in binary form must reproduce the above copyright
@@ -15,7 +16,7 @@
 # 3. Neither the name of The MacPorts Project nor the names of its
 #    contributors may be used to endorse or promote products derived from
 #    this software without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 # "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 # LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -27,15 +28,15 @@
 # THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-# 
-# 
+#
+#
 # This PortGroup lets a port check that the user's Xcode is sufficiently new.
-# 
+#
 # Usage:
-# 
+#
 #   PortGroup               xcodeversion 1.0
 #   minimum_xcodeversions   {darwin_major minimum_xcodeversion}
-# 
+#
 # where darwin_major is the major version of the underlying Darwin OS (e.g. 9
 # for Mac OS X 10.5 Leopard) and minimum_xcodeversion is the minimum version
 # of Xcode the port requires (e.g. 3.1).
@@ -43,13 +44,21 @@
 options minimum_xcodeversions
 default minimum_xcodeversions {}
 
-pre-extract {
-    if {"darwin" == ${os.platform}} {
-        set current_xcodeversion [exec defaults read ${developer_dir}/Applications/Xcode.app/Contents/Info CFBundleShortVersionString]
+platform macosx {
+    pre-extract {
         foreach {darwin_major minimum_xcodeversion} [join ${minimum_xcodeversions}] {
             if {${darwin_major} == ${os.major}} {
-                if {[rpm-vercomp ${current_xcodeversion} ${minimum_xcodeversion}] < 0} {
-                    ui_msg "On Mac OS X ${macosx_version}, ${name} ${version} requires Xcode ${minimum_xcodeversion} or later but you have Xcode ${current_xcodeversion}."
+                if {![info exists xcodeversion] || $xcodeversion == "none"} {
+                    ui_error "Couldn't determine your Xcode version (from '/usr/bin/xcodebuild -version')."
+                    ui_error ""
+                    ui_error "If you have not installed Xcode, install it now; see:"
+                    ui_error "http://guide.macports.org/chunked/installing.xcode.html"
+                    ui_error ""
+                    return -code error "unable to find Xcode"
+                }
+                if {[vercmp ${xcodeversion} ${minimum_xcodeversion}] < 0} {
+                    ui_error "On Mac OS X ${macosx_version}, ${name} ${version} requires Xcode ${minimum_xcodeversion} or later but you have Xcode ${xcodeversion}."
+                    ui_error "See http://guide.macports.org/chunked/installing.xcode.html for download links."
                     return -code error "incompatible Xcode version"
                 }
             }
