@@ -1,7 +1,7 @@
 <?php
     /* -*- coding: utf-8; mode: php; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:set fenc=utf-8 filetype=php et sw=4 ts=4 sts=4: */
-    /* Copyright (c) 2007, The MacPorts Project. */
-    /* $Id: index.php 80941 2011-07-21 21:43:01Z jmr@macports.org $ */
+    /* Copyright (c) 2007, 2014, The MacPorts Project. */
+    /* $Id: index.php 119247 2014-04-21 16:09:57Z cal@macports.org $ */
     include_once("includes/common.inc");
     print_header('The MacPorts Project -- Home', 'utf-8');
     $portsdb_info = portsdb_connect($portsdb_host, $portsdb_user, $portsdb_passwd);
@@ -13,15 +13,16 @@
     <h2 class="hdr">The MacPorts Project Official Homepage</h2>
 
     <p>The MacPorts Project is an open-source community initiative to design an easy-to-use system for compiling, installing,
-    and upgrading either command-line, X11 or Aqua based open-source software on the <a href="http://www.apple.com/macosx/">
-    Mac OS X</a> operating system. To that end we provide the command-line driven MacPorts software package under a <a
-    href="http://opensource.org/licenses/bsd-license.php">BSD License</a>, and through it easy access to thousands of ports
+    and upgrading either command-line, X11 or Aqua based open-source software on the <a href="http://www.apple.com/osx/">
+    OS X</a> operating system. To that end we provide the command-line driven MacPorts software package under a <a
+    href="http://opensource.org/licenses/BSD-3-Clause">BSD 3-Clause License</a>, and through it easy access to thousands of ports
     that <a href="<?php print $guide_url . '#introduction'; ?>"> greatly simplify</a> the task of <a href="<?php print $guide_url
     . '#using'; ?>">compiling and installing</a> open-source software on your Mac.</p>
     
     <p>We provide a single software tree that attempts to track the latest release of every software title (port) we distribute,
-    without splitting them into &#8220;stable&#8221; Vs. &#8220;unstable&#8221; branches, targeting mainly the current Mac OS
-    X release (10.7, A.K.A Lion) and the immediately previous one (10.6, A.K.A. Snow Leopard). There are currently <a href="ports.php">
+    without splitting them into &#8220;stable&#8221; Vs. &#8220;unstable&#8221; branches, targeting mainly the current OS
+    X release (OS X 10.9 Mavericks) and the immediately previous two (OS X 10.8 Mountain Lion and Mac OS X 10.7 Lion).
+    There are currently <a href="ports.php">
     <b><?php print $portsdb_info['num_ports']; ?></b> ports</a> in our tree, distributed among <?php print $portsdb_info['num_categories']; ?>
     different categories, and more are being added on a regular basis.</p>
     
@@ -41,6 +42,14 @@
     
     <h3 class="subhdr">Getting involved</h3>
     
+    <?php
+    /*
+    <p>We are participating in <b>Google Summer of Code <?php print date('Y'); ?></b>: MacPorts-wielding students of higher education can be paid
+    to work on open source software. <a href="<?php print $trac_url . 'wiki/SummerOfCode'; ?>">Head over to our ideas page</a> to see
+    if MacPorts is an interesting project for you.</p>
+    */
+    ?>
+
     <p>There are many ways you can get involved with MacPorts and peer users, system administrators &amp; developers alike.
     Browse over to the &#8220;<a href="contact.php">Contact Us</a>&#8221; section of our site and:</p>
     
